<?php

namespace Drupal\flipping_book\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\file\Plugin\Field\FieldType\FileItem;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;

/**
 * Plugin implementation of the 'flipping_book_link_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "flipping_book_link_formatter",
 *   label = @Translation("Flipping Book Link"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class FlippingBookLinkFormatter extends LinkFormatter {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements['url_only']['#access'] = TRUE;

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $entity = $items->getEntity();

    foreach ($items as $delta => $item) {
      $items[$delta]->title = $entity->getName();
    }

    return parent::viewElements($items, $langcode);
  }

  /**
   * {@inheritdoc}
   */
  protected function buildUrl(FileItem $item) {
    $flipping_book = $item->getRoot()->getValue();

    /** @var Url $url */
    $url = \Drupal::service('flipping_book')
      ->buildFlippingBookUrl($flipping_book);

    $settings = $this->getSettings();
    $options = $url->getOptions();

    // Add optional 'rel' attribute to link options.
    if (!empty($settings['rel'])) {
      $options['attributes']['rel'] = $settings['rel'];
    }
    // Add optional 'target' attribute to link options.
    if (!empty($settings['target'])) {
      $options['attributes']['target'] = $settings['target'];
    }
    $url->setOptions($options);

    return $url;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return (
      parent::isApplicable($field_definition) &&
      ($field_definition->getTargetEntityTypeId() == 'flipping_book') &&
      ($field_definition->getName() == 'file')
    );
  }

}
